import { Canvas, CanvasRenderingContext2D } from 'canvas';
import { BackgroundRankColor, Color, TextCard, UserStatus } from '../../../interface/card.interface';
export interface RankCardParams {
    nicknameText: TextCard;
    currentLvl: number;
    currentRank: number;
    currentXP: number;
    requiredXP: number;
    userStatus: UserStatus;
    currentXPColor?: Color;
    requiredXPColor?: Color;
    backgroundImgURL?: string;
    backgroundColor?: BackgroundRankColor;
    avatarImgURL?: string;
    avatarBackgroundColor?: Color;
    avatarBackgroundEnable?: boolean;
    progressBarColor?: Color;
    fontDefault?: string;
    colorTextDefault?: Color;
    lvlPrefix?: Omit<TextCard, 'content'> & Partial<Pick<TextCard, 'content'>>;
    lvlNumFormat?: Omit<TextCard, 'content'>;
    rankPrefix?: Omit<TextCard, 'content'> & Partial<Pick<TextCard, 'content'>>;
    rankNumFormat?: Omit<TextCard, 'content'>;
}
type OptionsDraw = {
    only?: string[];
    objectFit?: 'fill' | 'cover';
};
export declare class RankCardBuilder {
    nicknameText: TextCard;
    currentLvl: number;
    currentRank: number;
    currentXP: number;
    requiredXP: number;
    userStatus: UserStatus;
    backgroundImgURL?: string;
    backgroundColor: BackgroundRankColor;
    avatarImgURL?: string;
    avatarBackgroundColor: Color;
    avatarBackgroundEnable: boolean;
    fontDefault: string;
    colorTextDefault: Color;
    progressBarColor: Color;
    currentXPColor: Color;
    requiredXPColor: Color;
    lvlPrefix?: Omit<TextCard, 'content'> & Partial<Pick<TextCard, 'content'>>;
    rankPrefix?: Omit<TextCard, 'content'> & Partial<Pick<TextCard, 'content'>>;
    lvlNumFormat?: Omit<TextCard, 'content'>;
    rankNumFormat?: Omit<TextCard, 'content'>;
    constructor({ nicknameText, currentLvl, currentRank, currentXP, requiredXP, userStatus, backgroundImgURL, backgroundColor, avatarImgURL, avatarBackgroundColor, avatarBackgroundEnable, fontDefault, colorTextDefault, progressBarColor, currentXPColor, requiredXPColor, lvlPrefix, rankPrefix, lvlNumFormat, rankNumFormat, }: RankCardParams);
    setBackgroundColor(backgroundColor: BackgroundRankColor): this;
    setBackgroundImgURL(backgroundImgURL: string): this;
    setAvatarImgURL(avatarImgURL: string): this;
    setAvatarBackgroundColor(avatarBackgroundColor: Color): this;
    setAvatarBackgroundEnable(avatarBackgroundEnable: boolean): this;
    setFontDefault(fontDefault: string): this;
    setColorTextDefault(colorTextDefault: Color): this;
    setLvlPrefix(lvlPrefix: TextCard): this;
    setrankNumFormat(rankNumFormat: Omit<TextCard, 'content'>): this;
    setlvlNumFormat(lvlNumFormat: Omit<TextCard, 'content'>): this;
    setRankPrefix(rankPrefix: TextCard): this;
    setNicknameText(nicknameText: TextCard): this;
    setCurrentLvl(currentLvl: number): this;
    setCurrentRank(currentRank: number): this;
    setCurrentXP(currentXP: number): this;
    setRequiredXP(requiredXP: number): this;
    draw(ctx: CanvasRenderingContext2D, canvasWidth: number, canvasHeight: number, options?: OptionsDraw): Promise<void>;
    build(options?: OptionsDraw): Promise<Canvas>;
}
export {};
