"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RankCardBuilder = void 0;
const canvas_1 = require("canvas");
const hex_rgba_1 = require("../../../utils/hex-rgba");
const load_image_1 = require("../../../utils/load-image");
class RankCardBuilder {
    nicknameText;
    currentLvl;
    currentRank;
    currentXP;
    requiredXP;
    userStatus;
    backgroundImgURL;
    backgroundColor;
    avatarImgURL;
    avatarBackgroundColor;
    avatarBackgroundEnable;
    fontDefault;
    colorTextDefault;
    progressBarColor;
    currentXPColor;
    requiredXPColor;
    lvlPrefix;
    rankPrefix;
    lvlNumFormat;
    rankNumFormat;
    constructor({ nicknameText, currentLvl, currentRank, currentXP, requiredXP, userStatus, backgroundImgURL, backgroundColor = { background: '#FFF', bubbles: '#0CA7FF' }, avatarImgURL, avatarBackgroundColor = '#0CA7FF', avatarBackgroundEnable = true, fontDefault = 'Nunito', colorTextDefault = '#0CA7FF', progressBarColor = '#0CA7FF', currentXPColor = '#0CA7FF', requiredXPColor = '#7F8384', lvlPrefix, rankPrefix, lvlNumFormat, rankNumFormat, }) {
        this.nicknameText = nicknameText;
        this.currentLvl = currentLvl;
        this.currentRank = currentRank;
        this.currentXP = currentXP;
        this.requiredXP = requiredXP;
        this.userStatus = userStatus;
        this.backgroundImgURL = backgroundImgURL;
        this.backgroundColor = backgroundColor;
        this.avatarImgURL = avatarImgURL;
        this.avatarBackgroundColor = avatarBackgroundColor;
        this.avatarBackgroundEnable = avatarBackgroundEnable;
        this.fontDefault = fontDefault;
        this.colorTextDefault = colorTextDefault;
        this.progressBarColor = progressBarColor;
        this.currentXPColor = currentXPColor;
        this.requiredXPColor = requiredXPColor;
        this.lvlPrefix = lvlPrefix;
        this.rankPrefix = rankPrefix;
        this.lvlNumFormat = lvlNumFormat;
        this.rankNumFormat = rankNumFormat;
    }
    setBackgroundColor(backgroundColor) {
        this.backgroundColor = backgroundColor;
        return this;
    }
    setBackgroundImgURL(backgroundImgURL) {
        this.backgroundImgURL = backgroundImgURL;
        return this;
    }
    setAvatarImgURL(avatarImgURL) {
        this.avatarImgURL = avatarImgURL;
        return this;
    }
    setAvatarBackgroundColor(avatarBackgroundColor) {
        this.avatarBackgroundColor = avatarBackgroundColor;
        return this;
    }
    setAvatarBackgroundEnable(avatarBackgroundEnable) {
        this.avatarBackgroundEnable = avatarBackgroundEnable;
        return this;
    }
    setFontDefault(fontDefault) {
        this.fontDefault = fontDefault;
        return this;
    }
    setColorTextDefault(colorTextDefault) {
        this.colorTextDefault = colorTextDefault;
        return this;
    }
    setLvlPrefix(lvlPrefix) {
        this.lvlPrefix = lvlPrefix;
        return this;
    }
    setrankNumFormat(rankNumFormat) {
        this.rankNumFormat = rankNumFormat;
        return this;
    }
    setlvlNumFormat(lvlNumFormat) {
        this.lvlNumFormat = lvlNumFormat;
        return this;
    }
    setRankPrefix(rankPrefix) {
        this.rankPrefix = rankPrefix;
        return this;
    }
    setNicknameText(nicknameText) {
        this.nicknameText = nicknameText;
        return this;
    }
    setCurrentLvl(currentLvl) {
        this.currentLvl = currentLvl;
        return this;
    }
    setCurrentRank(currentRank) {
        this.currentRank = currentRank;
        return this;
    }
    setCurrentXP(currentXP) {
        this.currentXP = currentXP;
        return this;
    }
    setRequiredXP(requiredXP) {
        this.requiredXP = requiredXP;
        return this;
    }
    async draw(ctx, canvasWidth, canvasHeight, options) {
        if (!options?.only || options.only.includes('background')) {
            ctx.save();
            ctx.beginPath();
            ctx.moveTo(1000, 250);
            ctx.arcTo(0, 250, 0, 0, 30);
            ctx.arcTo(0, 0, 1000, 0, 30);
            ctx.arcTo(1000, 0, 1000, 250, 30);
            ctx.arcTo(1000, 250, 0, 250, 30);
            ctx.clip();
            if (this.backgroundImgURL) {
                try {
                    const img = await (0, canvas_1.loadImage)(this.backgroundImgURL);
                    if (options?.objectFit === 'cover') {
                        let offsetX = 0.5;
                        let offsetY = 0.5;
                        if (offsetX < 0)
                            offsetX = 0;
                        if (offsetY < 0)
                            offsetY = 0;
                        if (offsetX > 1)
                            offsetX = 1;
                        if (offsetY > 1)
                            offsetY = 1;
                        let iw = img.width, ih = img.height, r = Math.min(canvasWidth / iw, canvasHeight / ih), nw = iw * r, nh = ih * r, cx, cy, cw, ch, ar = 1;
                        if (nw < canvasWidth)
                            ar = canvasWidth / nw;
                        if (Math.abs(ar - 1) < 1e-14 && nh < canvasHeight)
                            ar = canvasHeight / nh;
                        nw *= ar;
                        nh *= ar;
                        cw = iw / (nw / canvasWidth);
                        ch = ih / (nh / canvasHeight);
                        cx = (iw - cw) * offsetX;
                        cy = (ih - ch) * offsetY;
                        if (cx < 0)
                            cx = 0;
                        if (cy < 0)
                            cy = 0;
                        if (cw > iw)
                            cw = iw;
                        if (ch > ih)
                            ch = ih;
                        ctx.drawImage(img, cx, cy, cw, ch, 0, 0, canvasWidth, canvasHeight);
                    }
                    else {
                        ctx.drawImage(img, 0, 0, canvasWidth, canvasHeight);
                    }
                }
                catch (err) {
                    throw new Error('Error loading the background image. The URL may be invalid.');
                }
            }
            else {
                ctx.fillStyle = this.backgroundColor.background;
                ctx.fillRect(0, 0, canvasWidth, canvasHeight);
                if (this.backgroundColor.bubbles) {
                    ctx.beginPath();
                    ctx.arc(153, 225, 10, 0, Math.PI * 2);
                    ctx.fillStyle = (0, hex_rgba_1.hexToRgbA)(this.backgroundColor.bubbles, 0.31);
                    ctx.fill();
                    ctx.closePath();
                    ctx.arc(213, 81, 10, 0, Math.PI * 2);
                    ctx.fillStyle = (0, hex_rgba_1.hexToRgbA)(this.backgroundColor.bubbles, 0.07);
                    ctx.fill();
                    ctx.closePath();
                    ctx.arc(238, 16, 10, 0, Math.PI * 2);
                    ctx.fillStyle = (0, hex_rgba_1.hexToRgbA)(this.backgroundColor.bubbles, 0.6);
                    ctx.fill();
                    ctx.closePath();
                    ctx.beginPath();
                    ctx.arc(486, 148, 40, 0, Math.PI * 2);
                    ctx.fillStyle = (0, hex_rgba_1.hexToRgbA)(this.backgroundColor.bubbles, 0.1);
                    ctx.fill();
                    ctx.closePath();
                    ctx.beginPath();
                    ctx.arc(396.5, 33.5, 7.5, 0, Math.PI * 2);
                    ctx.fillStyle = (0, hex_rgba_1.hexToRgbA)(this.backgroundColor.bubbles, 0.05);
                    ctx.fill();
                    ctx.closePath();
                    ctx.beginPath();
                    ctx.arc(515.5, 38.5, 12.5, 0, Math.PI * 2);
                    ctx.fillStyle = (0, hex_rgba_1.hexToRgbA)(this.backgroundColor.bubbles, 0.43);
                    ctx.fill();
                    ctx.closePath();
                    ctx.beginPath();
                    ctx.arc(572, 257, 30, 0, Math.PI * 2);
                    ctx.fillStyle = (0, hex_rgba_1.hexToRgbA)(this.backgroundColor.bubbles, 1);
                    ctx.fill();
                    ctx.closePath();
                    ctx.beginPath();
                    ctx.arc(782.5, 226.5, 8.5, 0, Math.PI * 2);
                    ctx.fillStyle = (0, hex_rgba_1.hexToRgbA)(this.backgroundColor.bubbles, 0.15);
                    ctx.fill();
                    ctx.closePath();
                    ctx.beginPath();
                    ctx.arc(1000, 101, 10, 0, Math.PI * 2);
                    ctx.fillStyle = (0, hex_rgba_1.hexToRgbA)(this.backgroundColor.bubbles, 0.63);
                    ctx.fill();
                    ctx.closePath();
                }
            }
            ctx.restore();
        }
        if (!options?.only || options.only.includes('avatarBorder')) {
            if (this.avatarBackgroundEnable) {
                ctx.beginPath();
                ctx.arc(88, 101, 75, 0, Math.PI * 2);
                ctx.fillStyle = this.avatarBackgroundColor;
                ctx.fill();
                ctx.closePath();
            }
        }
        if (!options?.only ||
            options.only.includes('avatar') ||
            options.only.includes('avatarBorder')) {
            if (this.avatarImgURL) {
                ctx.beginPath();
                ctx.arc(105, 125, 75, 0, Math.PI * 0.36, true);
                ctx.arc(159, 179, 23.5, Math.PI * 0.82, Math.PI * 1.68, false);
                ctx.arc(105, 125, 75, Math.PI * 0.15, Math.PI * 1.5, true);
                ctx.closePath();
                ctx.save();
                ctx.clip();
                const img = await (0, load_image_1.loadImageSafe)(this.avatarImgURL);
                if (!img) {
                    throw new Error('Error loading the background image. The URL may be invalid.');
                }
                ctx.drawImage(img, 30, 50, 150, 150);
                ctx.restore();
                ctx.beginPath();
                if (this.userStatus === 'online') {
                    ctx.arc(159, 179, 17, 0, Math.PI * 2);
                    ctx.fillStyle = '#57F287';
                }
                else if (this.userStatus === 'idle') {
                    ctx.arc(159, 179, 17, Math.PI * 0.9, Math.PI * 1.6, true);
                    ctx.arc(148, 168, 17, Math.PI * 1.9, Math.PI * 0.6);
                    ctx.fillStyle = '#faa61a';
                }
                else if (this.userStatus === 'dnd') {
                    ctx.arc(151, 179, 3.5, Math.PI * 1.5, Math.PI * 0.5, true);
                    ctx.arc(167, 179, 3.5, Math.PI * 0.5, Math.PI * 1.5, true);
                    ctx.closePath();
                    ctx.arc(159, 179, 17, 0, Math.PI * 2);
                    ctx.fillStyle = '#ed4245';
                }
                else if (this.userStatus === 'streaming') {
                    ctx.moveTo(168, 179);
                    ctx.lineTo(154.5, 170);
                    ctx.lineTo(154.5, 188);
                    ctx.closePath();
                    ctx.arc(159, 179, 17, 0, Math.PI * 2);
                    ctx.fillStyle = '#593695';
                }
                else {
                    ctx.arc(159, 179, 9, Math.PI * 1.5, Math.PI * 0.5, true);
                    ctx.arc(159, 179, 9, Math.PI * 0.5, Math.PI * 1.5, true);
                    ctx.closePath();
                    ctx.arc(159, 179, 17, 0, Math.PI * 2);
                    ctx.fillStyle = '#747f8d';
                }
                ctx.fill();
            }
        }
        if (!options?.only || options.only.includes('progressBar')) {
            ctx.save();
            ctx.beginPath();
            ctx.globalAlpha = 0.5;
            ctx.fillStyle = this.progressBarColor;
            ctx.arc(canvasWidth - 47.5, 182.5, 17.5, Math.PI * 1.5, Math.PI * 0.5);
            ctx.arc(227.5, 182.5, 17.5, Math.PI * 0.5, Math.PI * 1.5);
            ctx.fill();
            ctx.clip();
            ctx.closePath();
            const currentPercentXP = Math.floor((this.currentXP / this.requiredXP) * 100);
            if (currentPercentXP >= 1) {
                ctx.beginPath();
                const onePercentBar = (canvasWidth - 30 - 210) / 100;
                const pxBar = onePercentBar * currentPercentXP;
                ctx.globalAlpha = 1;
                ctx.fillStyle = this.progressBarColor;
                ctx.arc(192.5 + pxBar, 182.5, 17.5, Math.PI * 1.5, Math.PI * 0.5);
                ctx.arc(227.5, 182.5, 17.5, Math.PI * 0.5, Math.PI * 1.5);
                ctx.fill();
                ctx.closePath();
            }
            ctx.restore();
        }
        let offsetLvlXP = canvasWidth - 30;
        if (!options?.only || options.only.includes('xp')) {
            ctx.save();
            ctx.font = `600 35px '${this.fontDefault}'`;
            ctx.textAlign = 'right';
            ctx.fillStyle = this.requiredXPColor;
            ctx.fillText(`${this.requiredXP} xp`, offsetLvlXP, 150);
            offsetLvlXP -= ctx.measureText(`${this.requiredXP} xp`).width + 3;
            ctx.fillText('/', offsetLvlXP, 150);
            ctx.fillStyle = this.currentXPColor;
            offsetLvlXP -= ctx.measureText(`/`).width + 3;
            ctx.fillText(`${this.currentXP}`, offsetLvlXP, 150);
            offsetLvlXP -= ctx.measureText(`${this.currentXP}`).width;
            ctx.restore();
        }
        if (!options?.only || options.only.includes('nickname')) {
            const font = this.nicknameText.font ? this.nicknameText.font : this.fontDefault;
            const fontWeight = this.nicknameText.weight ? this.nicknameText.weight : '600';
            const fontSize = this.nicknameText.size ? this.nicknameText.size : 35;
            ctx.font = `${fontWeight} ${fontSize}px '${font}'`;
            ctx.fillStyle = this.nicknameText.color ? this.nicknameText.color : this.colorTextDefault;
            ctx.fillText(this.nicknameText.content, 210, 150, offsetLvlXP - 210 - 15);
        }
        ctx.save();
        let offsetRankX = canvasWidth - 30;
        if (!options?.only || options.only.includes('rank')) {
            ctx.textAlign = 'right';
            const rankContent = this.rankPrefix?.content ? this.rankPrefix.content : 'RANK';
            const fontNumber = this.rankNumFormat?.font ? this.rankNumFormat.font : this.fontDefault;
            const fontNumberSize = this.rankNumFormat?.size ? this.rankNumFormat.size : 60;
            const fontNumberWeight = this.rankNumFormat?.weight ? this.rankNumFormat.weight : '600';
            ctx.fillStyle = this.rankNumFormat?.color ? this.rankNumFormat.color : this.colorTextDefault;
            ctx.font = `${fontNumberWeight} ${fontNumberSize}px '${fontNumber}'`;
            ctx.fillText(`${this.currentRank}`, offsetRankX, 75);
            offsetRankX -= ctx.measureText(`${this.currentRank}`).width;
            const fontPrefix = this.rankPrefix?.font ? this.rankPrefix.font : this.fontDefault;
            const fontPrefixSize = this.rankPrefix?.size ? this.rankPrefix.size : 35;
            const fontPrefixWeight = this.rankPrefix?.weight ? this.rankPrefix.weight : '600';
            ctx.fillStyle = this.rankPrefix?.color ? this.rankPrefix.color : this.colorTextDefault;
            ctx.font = `${fontPrefixWeight} ${fontPrefixSize}px '${fontPrefix}'`;
            ctx.fillText(` ${rankContent} `, offsetRankX, 75);
            offsetRankX -= ctx.measureText(` ${rankContent} `).width;
        }
        if (!options?.only || options.only.includes('lvl')) {
            const lvlContent = this.lvlPrefix && this.lvlPrefix.content ? this.lvlPrefix.content : 'LVL';
            ctx.fillStyle =
                this.lvlPrefix && this.lvlPrefix.color ? this.lvlPrefix.color : this.colorTextDefault;
            const fontNumber = this.lvlNumFormat?.font ? this.lvlNumFormat.font : this.fontDefault;
            const fontNumberSize = this.lvlNumFormat?.size ? this.lvlNumFormat.size : 60;
            const fontNumberWeight = this.lvlNumFormat?.weight ? this.lvlNumFormat.weight : '600';
            ctx.fillStyle = this.lvlNumFormat?.color ? this.lvlNumFormat.color : this.colorTextDefault;
            ctx.font = `${fontNumberWeight} ${fontNumberSize}px '${fontNumber}'`;
            ctx.fillText(`${this.currentLvl}`, offsetRankX, 75);
            offsetRankX -= ctx.measureText(`${this.currentLvl}`).width;
            const fontPrefix = this.lvlPrefix?.font ? this.lvlPrefix.font : this.fontDefault;
            const fontPrefixSize = this.lvlPrefix?.size ? this.lvlPrefix.size : 35;
            const fontPrefixWeight = this.lvlPrefix?.weight ? this.lvlPrefix.weight : '600';
            ctx.fillStyle = this.lvlPrefix?.color ? this.lvlPrefix.color : this.colorTextDefault;
            ctx.font = `${fontPrefixWeight} ${fontPrefixSize}px '${fontPrefix}'`;
            ctx.fillText(`${lvlContent} `, offsetRankX, 75);
            ctx.restore();
        }
    }
    async build(options) {
        const canvas = (0, canvas_1.createCanvas)(1000, 250);
        const ctx = canvas.getContext('2d');
        await this.draw(ctx, canvas.width, canvas.height, options);
        return canvas;
    }
}
exports.RankCardBuilder = RankCardBuilder;
